package atomicwheel;

import java.util.Arrays;
import java.util.Collection;

/**
 * A very simple, immutable, directed or undirected Edge.
 * 
 * @author matthias
 */
public class Edge extends GraphItem {
	private static final long serialVersionUID = 1L;

	final Node source;
	final Node target;
	final boolean directed;

	/**
	 * Creates an edge.
	 * 
	 * @param source
	 *            source node
	 * @param target
	 *            target node
	 * @param directed
	 *            if <code>true</code>, the order of {@code source} and
	 *            {@code target} matters
	 */
	public Edge(final Node source, final Node target, final boolean directed) {
		if (source == null || target == null)
			throw new NullPointerException("source or target node is null");
		this.source = source;
		this.target = target;
		this.directed = directed;
	}

	/** @return the source node */
	public Node getSource() {
		return source;
	}

	/** @return the target node */
	public Node getTarget() {
		return target;
	}

	/**
	 * @param node
	 *            one of the endnodes of this node
	 * @return the node opposite of the given node
	 */
	public Node getOpposite(final Node node) {
		if (node == target)
			return source;
		if (node == source)
			return target;
		throw new IllegalArgumentException(node + " is not an endnode of edge "
				+ this);
	}

	/** @return <code>true</code> if the edge is directed */
	public boolean isDirected() {
		return directed;
	}

	/** @return a {@link Collection} containing the two endnodes */
	public Collection<Node> getEndnodes() {
		return Arrays.asList(target, source);
	}
}
